      subroutine gamma ( x, ga )

c*********************************************************************72
c
cc GAMMA computes the Gamma function.
c
c  Licensing:
c
c    This routine is copyrighted by Shanjie Zhang and Jianming Jin.  However, 
c    they give permission to incorporate this routine into a user program 
c    provided that the copyright is acknowledged.
c
c  Modified:
c
c    14 July 2012
c
c  Author:
c
c    Shanjie Zhang, Jianming Jin
c
c  Reference:
c
c    Shanjie Zhang, Jianming Jin,
c    Computation of Special Functions,
c    Wiley, 1996,
c    ISBN: 0-471-11963-6,
c    LC: QA351.C45.
c
c  Parameters:
c
c    Input, double precision X, the argument.
c
c    Output, double precision GA, the function value.
c
      implicit none

      double precision g(26)
      double precision ga
      double precision gr
      integer k
      integer m
      integer m1
      double precision pi
      double precision r
      double precision x
      double precision z

      save g

      data g /
     &   1.0D+00, 
     &   0.5772156649015329D+00,
     &  -0.6558780715202538D+00, 
     &  -0.420026350340952D-01,
     &   0.1665386113822915D+00,
     &  -0.421977345555443D-01,
     &  -0.96219715278770D-02, 
     &   0.72189432466630D-02,
     &  -0.11651675918591D-02, 
     &  -0.2152416741149D-03,
     &   0.1280502823882D-03,
     &  -0.201348547807D-04,
     &  -0.12504934821D-05, 
     &   0.11330272320D-05,
     &  -0.2056338417D-06, 
     &   0.61160950D-08,
     &   0.50020075D-08, 
     &  -0.11812746D-08,
     &   0.1043427D-09, 
     &   0.77823D-11,
     &  -0.36968D-11, 
     &   0.51D-12,
     &  -0.206D-13, 
     &  -0.54D-14, 
     &   0.14D-14, 
     &   0.1D-15 /

      pi = 3.141592653589793D+00

      if ( x .eq. int ( x ) ) then

        if ( 0.0D+00 .lt. x ) then
          ga = 1.0D+00
          m1 = x - 1
          do k = 2, m1
            ga = ga * k
          end do
        else
          ga = 1.0D+300
        end if

      else

        if ( 1.0D+00 .lt. abs ( x ) ) then
          z = abs ( x )
          m = int ( z )
          r = 1.0D+00
          do k = 1, m
            r = r * ( z - k )
          end do
          z = z - m
        else
          z = x
        end if

        gr = g(26)
        do k = 25, 1, -1
          gr = gr * z + g(k)
        end do
        ga = 1.0D+00 / ( gr * z )

        if ( 1.0D+00 .lt. abs ( x ) ) then
          ga = ga * r
          if ( x .lt. 0.0D+00 ) then
            ga = - pi / ( x * ga * sin ( pi * x ) )
          end if
        end if

      end if

      return
      end
